import uuid
from typing import List, Optional

from sqlalchemy.orm import Session

from app.crud.base import CRUDBase
from app.models.tag import Tag
from app.schemas.tag import TagCreate, TagUpdate


class CRUDTag(CRUDBase[Tag, TagCreate, TagUpdate]):
    def get_by_name(self, db: Session, *, name: str) -> Optional[Tag]:
        """
        Get a tag by name.
        """
        return db.query(self.model).filter(Tag.name == name).first()

    def create(self, db: Session, *, obj_in: TagCreate) -> Tag:
        """
        Create a new tag.
        """
        tag_id = str(uuid.uuid4())
        db_obj = Tag(
            id=tag_id,
            name=obj_in.name,
            color=obj_in.color,
        )
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def get_multi_by_note(
        self, db: Session, *, note_id: str
    ) -> List[Tag]:
        """
        Get all tags for a specific note.
        """
        return (
            db.query(self.model)
            .join(Tag.notes)
            .filter(Tag.notes.any(id=note_id))
            .all()
        )

    def get_or_create(
        self, db: Session, *, name: str, color: str = "#FFFFFF"
    ) -> Tag:
        """
        Get a tag by name or create it if it doesn't exist.
        """
        tag = self.get_by_name(db, name=name)
        if tag:
            return tag
        
        tag_create = TagCreate(name=name, color=color)
        return self.create(db, obj_in=tag_create)


tag = CRUDTag(Tag)